from flask_restful import Resource
from lyrebird.mock import context
from flask import request
from lyrebird import version
from lyrebird import application


class Status(Resource):

    def get(self):
        conf = context.application.conf
        return context.make_ok_response(
            **{'ip': conf.get('ip', 'unknown ip'),
               'mock.port': conf['mock.port'],
               'proxy.port': conf['proxy.port'],
               'version': version.VERSION
               })


class Manifest(Resource):

    def get(self):
        conf = context.application.conf
        return context.make_ok_response(
            **{'manifest': conf.get('manifest', [])
               })


class WorkMode(Resource):

    def put(self, mode=None):
        if context.Mode.contains(mode):
            context.application.work_mode = mode
            return application.make_ok_response()
        else:
            return application.make_fail_response(f'Unknown record mode: {mode}')

    def get(self):
        return application.make_ok_response(data=context.application.work_mode)


class DiffMode(Resource):

    def put(self):
        is_diff_mode = request.json.get('status')
        context.application.is_diff_mode = is_diff_mode
        return application.make_ok_response()

    def get(self):
        return application.make_ok_response(diffmode=context.application.is_diff_mode)



from flask import Response


class DuplicateRequest:

    def handle(self, handler_context):
        headers = {
            'Content-Type': 'text/html; charset=utf-8'
        }
        code = 445
        resp_data = f'Duplicate request found: {handler_context.flow.get("url")}\n'

        handler_context.flow['response']['headers'] = headers
        handler_context.flow['response']['code'] = code
        handler_context.flow['response']['data'] = resp_data

        handler_context.response = Response(resp_data, status=code, headers=headers)



from pathlib import Path
from lyrebird.mock import context
from lyrebird.log import get_logger
from .mock_data_helper import MockDataHelper


logger = get_logger()


class MockHandler:
    """
    根据当前设置数据组的匹配条件,查找对应的mock数据。

    """

    def handle(self, handler_context):
        hit_datas = context.application.data_manager.get_matched_data(handler_context.flow)
        if len(hit_datas) <= 0:
            return

        # TODO rules of hitting multiple mock data
        hit_data = hit_datas[0]

        activated_groups = context.application.data_manager.activated_group
        activated_group = list(activated_groups.values())[0]
        logger.info(
            f'<Mock> Hit Group:{activated_group.get("name")} - Data:{hit_data["name"]} \nURL: {handler_context.flow["request"]["url"]}\nGroupID:{activated_group["id"]} DataID:{hit_data["id"]}')

        handler_context.flow['response']['code'] = hit_data['response']['code']
        handler_context.flow['response']['headers'] = {k:v for k,v in hit_data['response']['headers'].items()}
        handler_context.flow['response']['data'] = hit_data['response']['data']

        handler_context.set_response_edited()
        handler_context.set_response_source_mock()
        handler_context.flow['response']['headers']['isMocked'] = 'True'
        handler_context.flow['response']['headers']['lyrebird'] = 'mock'
        handler_context.add_flow_action({
            'id': 'mock',
            'name': Path(__file__).name,
            'mock_id': hit_data['id']
        })
        MockDataHelper.read_mock_data(handler_context)



from lyrebird import application
from .. import checker


class CustomEncoder:

    def __call__(self, rules=None, *args, **kw):
        def func(origin_func):
            func_type = checker.TYPE_ENCODER
            if not checker.scripts_tmp_storage.get(func_type):
                checker.scripts_tmp_storage[func_type] = []
            checker.scripts_tmp_storage[func_type].append({
                'name': origin_func.__name__,
                'func': origin_func,
                'rules': rules
            })
            return origin_func
        return func

    @staticmethod
    def register(func_info):
        application.encoder.append(func_info)

    @staticmethod
    def unregister(func_info):
        if func_info in application.encoder:
            application.encoder.remove(func_info)

encoder = CustomEncoder()



from lyrebird.plugins import ClickableStatusText, TextMenuItem, ImageMenuItem
import qrcode
import io
import binascii


class StatusbarDemoImage(ClickableStatusText):
    def get_text(self):
        return 'Lyrebird_QRCode'

    def get_menu(self):

        img = qrcode.make("https://github.com/Meituan-Dianping/lyrebird")
        img_buffer = io.BytesIO()
        img.save(img_buffer, 'png')
        img_data_bytes = img_buffer.getvalue()
        img_buffer.close()

        img_data = (b"data:image/png;base64," + binascii.b2a_base64(img_data_bytes)).decode()
        return [ImageMenuItem(src=img_data)]

class StatusbarDemoText(ClickableStatusText):
    def get_text(self):
        return 'Lyrebird_Text'

    def get_menu(self):
        return [TextMenuItem(src="Text Status Bar")]



import uuid


class ClickableStatusText:
    """
    Plugin status text component

    On main UI rendering , Lyrebird will call getText function and display its return on footbar.
    If user click that text， Lyrebird will call getMenu function for loading menu items.
    """

    def __init__(self):
        self.id = str(uuid.uuid4())

    def get_text(self):
        """
        return a string
        """
        pass

    def get_menu(self):
        """
        return a list of MenuItem

        e.g.
        return [TextMenuItem('Hello'), TextMenuItem('World')]
        """
        pass


class MenuItem:

    def __init__(self, src):
        self.id = str(uuid.uuid4())
        self.type = self.__class__.__name__
        self.src = src

    def json(self):
        return {'id': self.id, 'type': self.type, 'src': self.src}


class ImageMenuItem(MenuItem):
    """
    The src argument support URL and base64 data string
    """
    pass


class TextMenuItem(MenuItem):
    """
    src: text
    """
    pass



from collections import OrderedDict
from . import content_encoding, content_type

origin2flow_handlers = OrderedDict({
    'Content-Encoding': content_encoding,
    'Content-Type': content_type
})

flow2origin_handlers = OrderedDict({
    'Content-Type': content_type,
    'Content-Encoding': content_encoding
})


class DataHelper:

    @staticmethod
    def origin2flow(origin_obj, output=None):
        _data = origin_obj.data
        if not _data:
            return

        for headers_key, func in origin2flow_handlers.items():
            headers_val = origin_obj.headers.get(headers_key, '')
            _data = func.origin2flow(headers_val, _data)

        if output:
            output['data'] = _data
        else:
            return _data

    @staticmethod
    def flow2origin(flow_obj, output=None):
        _data = flow_obj.get('data')
        if not _data:
            return

        for headers_key, func in flow2origin_handlers.items():
            headers_val = flow_obj['headers'].get(headers_key, '')
            _data = func.flow2origin(headers_val, _data)

        if output:
            output.data = _data
        else:
            return _data



"""
This file contains python3.6+ syntax!
Feel free to import and use whatever new package you deem necessary.
"""

import os
import sys
import asyncio
import argparse  # noqa
import signal  # noqa
import typing  # noqa

from mitmproxy.tools import cmdline  # noqa
from mitmproxy import exceptions, master  # noqa
from mitmproxy import options  # noqa
from mitmproxy import optmanager  # noqa
from mitmproxy import proxy  # noqa
from mitmproxy.utils import debug, arg_check  # noqa

OPTIONS_FILE_NAME = "config.yaml"


def assert_utf8_env():
    spec = ""
    for i in ["LANG", "LC_CTYPE", "LC_ALL"]:
        spec += os.environ.get(i, "").lower()
    if "utf" not in spec:
        print(
            "Error: mitmproxy requires a UTF console environment.",
            file=sys.stderr
        )
        print(
            "Set your LANG environment variable to something like en_US.UTF-8",
            file=sys.stderr
        )
        sys.exit(1)


def process_options(parser, opts, args):
    if args.version:
        print(debug.dump_system_info())
        sys.exit(0)
    if args.quiet or args.options or args.commands:
        # also reduce log verbosity if --options or --commands is passed,
        # we don't want log messages from regular startup then.
        args.termlog_verbosity = 'error'
        args.flow_detail = 0
    if args.verbose:
        args.termlog_verbosity = 'debug'
        args.flow_detail = 2

    adict = {}
    for n in dir(args):
        if n in opts:
            adict[n] = getattr(args, n)
    opts.merge(adict)

    return proxy.config.ProxyConfig(opts)


def run(
        master_cls: typing.Type[master.Master],
        make_parser: typing.Callable[[options.Options], argparse.ArgumentParser],
        arguments: typing.Sequence[str],
        extra: typing.Callable[[typing.Any], dict] = None
) -> master.Master:  # pragma: no cover
    """
        extra: Extra argument processing callable which returns a dict of
        options.
    """
    
    # Remove debuger
    # debug.register_info_dumpers()
    
    # Add looper in server thread
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

    opts = options.Options()
    master = master_cls(opts)

    parser = make_parser(opts)

    # To make migration from 2.x to 3.0 bearable.
    if "-R" in sys.argv and sys.argv[sys.argv.index("-R") + 1].startswith("http"):
        print("-R is used for specifying replacements.\n"
              "To use mitmproxy in reverse mode please use --mode reverse:SPEC instead")

    try:
        args = parser.parse_args(arguments)
    except SystemExit:
        arg_check.check()
        sys.exit(1)
    try:
        opts.confdir = args.confdir
        optmanager.load_paths(
            opts,
            os.path.join(opts.confdir, OPTIONS_FILE_NAME),
        )
        pconf = process_options(parser, opts, args)
        server: typing.Any = None
        if pconf.options.server:
            try:
                server = proxy.server.ProxyServer(pconf)
            except exceptions.ServerException as v:
                print(str(v), file=sys.stderr)
                sys.exit(1)
        else:
            server = proxy.server.DummyServer(pconf)

        master.server = server
        if args.options:
            print(optmanager.dump_defaults(opts))
            sys.exit(0)
        if args.commands:
            master.commands.dump()
            sys.exit(0)
        opts.set(*args.setoptions, defer=True)
        if extra:
            opts.update(**extra(args))

        if os.name == "nt":
            async def wakeup():
                while True:
                    await asyncio.sleep(0.2)
            asyncio.ensure_future(wakeup())

        master.run()
    except exceptions.OptionsError as e:
        print("%s: %s" % (sys.argv[0], e), file=sys.stderr)
        sys.exit(1)
    except (KeyboardInterrupt, RuntimeError) as e:
        pass
    return master


def mitmproxy(args=None) -> typing.Optional[int]:  # pragma: no cover
    if os.name == "nt":
        print("Error: mitmproxy's console interface is not supported on Windows. "
              "You can run mitmdump or mitmweb instead.", file=sys.stderr)
        return 1
    assert_utf8_env()
    from mitmproxy.tools import console
    run(console.master.ConsoleMaster, cmdline.mitmproxy, args)
    return None


def mitmdump(args=None) -> typing.Optional[int]:  # pragma: no cover
    from mitmproxy.tools import dump

    def extra(args):
        if args.filter_args:
            v = " ".join(args.filter_args)
            return dict(
                save_stream_filter=v,
                readfile_filter=v,
                dumper_filter=v,
            )
        return {}

    m = run(dump.DumpMaster, cmdline.mitmdump, args, extra)
    if m and m.errorcheck.has_errored:  # type: ignore
        return 1
    return None


def mitmweb(args=None) -> typing.Optional[int]:  # pragma: no cover
    from mitmproxy.tools import web
    run(web.master.WebMaster, cmdline.mitmweb, args)
    return None
